#include <TinyGPS++.h>
#include <SoftwareSerial.h>
#include <SD.h>
const int chipSelect = 8;

static const int RXPin = 2, TXPin = 3;
static const uint32_t GPSBaud = 9600;

File dataFile;
char filename[] = "LOGGER00.CSV";

// The TinyGPS++ object
TinyGPSPlus gps;

// The serial connection to the GPS device
SoftwareSerial ss(RXPin, TXPin);

// For stats that happen every 5 seconds
unsigned long last = 0UL;

void setup()
{
  Serial.begin(9600);
  ss.begin(GPSBaud);

  Serial.print(F("Initializing SD card. "));
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(chipSelect, OUTPUT);
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println(F("Card failed."));
    // don't do anything more:
    return;
  }
  Serial.println(F("Card initialized."));
  
// create a new file with a new filenumber
for (uint8_t i = 0; i < 100; i++) {
    filename[6] = i/10 + '0';
    filename[7] = i%10 + '0';
      if (! SD.exists(filename)) {
      // only open a new file if it doesn't exist
      dataFile = SD.open(filename, FILE_WRITE);
      break; // leave the loop
      }
      dataFile.close();
  }
if (! dataFile) {
  Serial.println("couldnt create file");
  }
  Serial.print("Logging to: ");
  Serial.println(filename);
  Serial.println("");  
  
    //_________________________Write Log File Header__________________________________________________
File dataFile = SD.open(filename, FILE_WRITE); 
  if (dataFile)
  {                                 
    dataFile.print(F("Latitud"));          
    dataFile.print(",");
    dataFile.print(F("Altitud"));
    dataFile.print(",");
    dataFile.print(F("Velocidad"));
    dataFile.print(",");
    dataFile.print(F("Fecha"));
    dataFile.print(",");
    dataFile.print(F("Hora"));
    dataFile.print(",");
    dataFile.println(F("Distancia al objetivo"));
    dataFile.close();
  }
    Serial.print(F("Latitud"));          
    Serial.print(",");
    Serial.print(F("Longitud"));
    Serial.print(",");
    Serial.print(F("Velocidad"));
    Serial.print(",");
    Serial.print(F("Fecha"));
    Serial.print(",");
    Serial.print(F("Hora"));
    Serial.print(",");
    Serial.println(F("Distancia al objetivo"));
}

void loop()
{
  // Dispatch incoming characters
  while (ss.available() > 0)
    gps.encode(ss.read());
    
if (gps.location.isUpdated())
  {

    Serial.print(gps.location.lat(), 6);
    Serial.print(F(","));
    Serial.print(gps.location.lng(), 6);
    Serial.print(F(","));
    File dataFile = SD.open(filename, FILE_WRITE);
    dataFile.print(gps.location.lat(), 6);
    dataFile.print(F(","));
    dataFile.print(gps.location.lng(), 6);
    dataFile.print(F(","));
    dataFile.close();
  }
 

else if (gps.speed.isUpdated())
  {

    Serial.print(gps.speed.kmph());
    Serial.print(F(","));
    File dataFile = SD.open(filename, FILE_WRITE);
    dataFile.print(gps.speed.kmph());
    dataFile.print(F(","));
    dataFile.close();
  }
  
else if (gps.date.isUpdated())
  {

    Serial.print(gps.date.day());
    Serial.print(F("/"));
    Serial.print(gps.date.month());
    Serial.print(F("/"));
    Serial.print(gps.date.year());
    Serial.print(F(","));
    File dataFile = SD.open(filename, FILE_WRITE);
    if (gps.date.day()<10){  // Add a zero, if necessary, as above
     dataFile.print(0);
     }
    dataFile.print(gps.date.day());
    dataFile.print(F("."));
    
    if (gps.date.month()<10){  // Add a zero, if necessary, as above
     dataFile.print(0);
     }
    dataFile.print(gps.date.month());
    dataFile.print(F("."));
    
    dataFile.print(gps.date.year());
    dataFile.print(F(","));
    dataFile.close();
  }

else if (gps.time.isUpdated())
  {
    Serial.print(gps.time.hour());
    Serial.print(F(":"));
    Serial.print(gps.time.minute());
    Serial.print(F(":"));
    Serial.print(gps.time.second());
    Serial.print(F(","));
    File dataFile = SD.open(filename, FILE_WRITE);
    if (gps.time.hour()<10){  // Add a zero, if necessary, as above
     dataFile.print(0);
     }
    dataFile.print(gps.time.hour());
    dataFile.print(F(":"));
    
    if (gps.time.minute()<10){  // Add a zero, if necessary, as above
     dataFile.print(0);
     }
    dataFile.print(gps.time.minute());
    dataFile.print(F(":"));
    
    if (gps.time.second()<10){  // Add a zero, if necessary, as above
     dataFile.print(0);
     }
    dataFile.print(gps.time.second());
    dataFile.print(F(","));
    dataFile.close();
  }
  
  else if (millis() - last > 1000)
  {
//    Serial.println();
    if (gps.location.isValid())
    {
      static const double MADRID_LAT = 40.4893538, MADRID_LON = -3.6827461; //custom location
      double distanceToMadrid =
        TinyGPSPlus::distanceBetween(
          gps.location.lat(),
          gps.location.lng(),
          MADRID_LAT, 
          MADRID_LON);
      double courseToLondon =
        TinyGPSPlus::courseTo(
          gps.location.lat(),
          gps.location.lng(),
          MADRID_LAT, 
          MADRID_LON);
          
File dataFile = SD.open(filename, FILE_WRITE);
    dataFile.println(distanceToMadrid/1000, 6);
    dataFile.close();
    Serial.println(distanceToMadrid/1000, 6);
    }

    last = millis();

  }
}
